--[[
	GroupTargets
	Version 7.2a
	Last Updated:   May 9, 2017

	Author: Rhianidd of Silver Hand (rhianidd@silverdaggers.net)
	Website: http://www.wowinterface.com/downloads/info16413-RhianiddsGroupTargets.html

	NB: The internal module model offers no significant savings in memory when enabled/disabled, so the user pays most
			of the cost even when not running; switching to a user-select model offers the user flexibility though at 
			the cost of a more cluttered addon list while letting the configuration module remain LoadOnDemand.

vSDSH Notes: Maybe the unit-constructor/updator functions should be in a function table so that instead of many 
copy/paste process-all-frames loops, things can be handled a bit differently:

Change This...
	local enabledFrames = ctg.enabledFrames
	for fName, _ in pairs(enabledFrames) do
		self.uFrames[fName] = self:makeFrame(fName, "gt_"..fName, _G[BlizzardFrameIndex[fName] ])
	end	

Into This...

funcStore = {
	["Bender_KillAllHumans"] = function(...)
		-- Insert Futurama Episode
	end
}

GT:ProcAllFrames(funcStore["Bender_KillAllHumans"], ...)

function GT:ProcAllFrames(func, ...)
	local enabledFrames = ctg.enabledFrames
	for fName, _ in pairs(enabledFrames) do
		func(...)
	end	
end

I can't recall if vars can be passed like this (have to look up the funcTable used in ExecAssist) but if not then a
var table (self.varTable={...}) could be recycled.
-- --------------------------------------------- --------------------------------------------- --------------------------- --]]

-- [ Libraries and Modules ] --------------------------------------------- --
local AddonName = ...
GroupTargets = LibStub("AceAddon-3.0"):NewAddon("GroupTargets", "AceEvent-3.0", "libPortableTools-1.0")
local debug = function(...) GroupTargets:debugAlert(...) end

-- [ local Vars ] -------------------------------------------------------- --
local BlizzardFrameIndex = { -- unit name to Blizzard Unit Frame index, since the Unit Frame names diverge so annoyingly from unit designations and masking is stilly for this few names
	["pet"]    			= "PetFrame",
	["party1"]			= "PartyMemberFrame1",
	["party2"] 			= "PartyMemberFrame2",
	["party3"] 			= "PartyMemberFrame3",
	["party4"] 			= "PartyMemberFrame4",
	["partypet1"] 	= "PartyMemberFrame1PetFrame",
	["partypet2"] 	= "PartyMemberFrame2PetFrame",
	["partypet3"] 	= "PartyMemberFrame3PetFrame",
	["partypet4"]   = "PartyMemberFrame4PetFrame",
}

local SetIndex = {
	["pet"]    			= "pet",
	["party1"]			= "party",
	["party2"] 			= "party",
	["party3"] 			= "party",
	["party4"] 			= "party",
	["partypet1"] 	= "partypet",
	["partypet2"] 	= "partypet",
	["partypet3"] 	= "partypet",
	["partypet4"]   = "partypet",
}

local function C(color, text) return "|cff"..colors[color]..(text or "[no data]").."|r" end

function GroupTargets:OnInitialize()
	self.moduleName_STYLEFRAME = "GroupTargets_StyleFrames"	-- moduleName Constant
	self:pt_Setup("GroupTargets", "GT", true) 							-- portable tools setup (debug/alert)
	
	-- uFrames is the handle pool for the created gt_frames as defined by GroupTargets_DB and indexed by unit such that
	-- uFrames[unit] returns the a frame handle or nil (if the requested frame wasn't created), also allows for agnostic
	-- looping structure
	self.uFrames = {}

	-- [ Setup Database ] -------------------------------------------------- --
	self.defaults = {
		["global"] = {
			["DisplayFirstUseMsg"] = true,
			["enabledModules"] = {},
			["enabledSets"]   = {}, -- see 'First Use Setup' for load
			["enabledFrames"] = {}, -- see 'First Use Setup' for load
			["DBIconTable"] = { ["hide"] = false },
			
			-- Constructor Options
			-- [""] = ,
			["RegisterUnitWatch"] = false,
			["color"] = {
				["Friendly"] = {["r"] = 0,  ["g"] = 1,  ["b"] = 0 }, -- "green"
				["Hostile"]  = {["r"] = 1,  ["g"] = 0,  ["b"] = 0 }, -- "red"
				["NoTarget"] = {["r"] = .8, ["g"] = .8, ["b"] = .8}, -- "gray"
				["Neutral"]  = {["r"] = 1,  ["g"] = 1,  ["b"] = 1 }, -- "white"
			},
			["defaults"] = {
				["width"] = 250,
				["height"] = 22,
				["fileBackdrop"] = "",
				["fileEdge"] = "",
				["bgColor"] = 0,
				["edgeColor"] = 0,

				["iconSide"] = "LEFT",
				["iconSize"] = 16,
				["icon_dx"] = 4,
				["icon_dy"] = 0,

				["fsLeftAnchor_dx"] = 4,
				["fsRightAnchor_dy"] = nil,
				["fsFont"] = "Fonts\\FRIZQT__.TTF",
				["fsFontSize"] = 12, 
				["fsFlags"] = "OUTLINE",
			},
			["pos"] = {
		   		["pet"]      = {["point"]="TOP",     ["relative"]="BOTTOM",   ["dx"]=10, ["dy"]=0},
					["party"]    = {["point"]="TOPLEFT", ["relative"]="TOPRIGHT", ["dx"]=15, ["dy"]=-25},
					["partypet"] = {["point"]="TOPLEFT", ["relative"]="TOPRIGHT", ["dx"]=15, ["dy"]=-25},
			},
		}
	}
	self.db = LibStub("AceDB-3.0"):New("GroupTargets_DB", self.defaults, true)	-- Setup DB
	self:addBlizzardOptionsPanel()                                              -- Setup Blizzard Options (stub)
	
	-- First Use Setup
	local ctg = self.db.global
	if ctg.DisplayFirstUseMsg then 
		ctg.enabledSets = {["party"] = true,["partypet"] = true};	ctg.enabledFrames = {["pet"] = true,["party1"] = true,["party2"] = true,["party3"] = true,["party4"] = true,["partypet1"] = true,["partypet2"] = true,["partypet3"] = true,["partypet4"] = true} 
		-- First-Time-Use Message (if any) -- self:alert("")
	end

	-- [ Setup Unit Frame Target Handles ] --------------------------------- --
	local enabledFrames = ctg.enabledFrames
	for fName, _ in pairs(enabledFrames) do
		self.uFrames[fName] = self:makeFrame(fName, "gt_"..fName, _G[BlizzardFrameIndex[fName]])
	end	
	GroupTargets:allUnits_Reposition()
	GroupTargets:allUnits_RegisterUnitWatch()
	-- --------------------------------- [ Setup Unit Frame Target Handles ] --
end
function GroupTargets:OnEnable()
	-- [ Register Events ] ------------------------------------------------- --	
	self:RegisterEvent("RAID_TARGET_UPDATE", "UpdateAllUnits") -- Fires When a Raid Target Icon is set/unset
 	self:RegisterEvent("UNIT_TARGET",				 "UpdateUnit") 		 -- Fires when a unit changes targets (var0=event, var1=unit)
	-- ------------------------------------------------- [ Register Events ] --

	-- [ First Use Msg ] ---------------------------------------------------- --
	local ctg = self.db.global
	if ctg.DisplayFirstUseMsg then
		self:alert(GetAddOnMetadata("GroupTargets","Title").." v"..GetAddOnMetadata("GroupTargets","Version").." loaded.\nOptions accessed by clicking on the Minimap Icon or DataBroker Object.")
		ctg.DisplayFirstUseMsg = false
	end		
	
	-- LDB / DBIcon
  self.LDB_Handle = self:ldb_ObjectCreate()
  LibStub("LibDBIcon-1.0"):Register(AddonName, self.LDB_Handle, self.db.global.DBIconTable) 
		
	self.loadedModules = {} -- may need to have an order-of-load table for modules
	for name, module in GroupTargets:IterateModules() do 
		self.loadedModules[name] = true
		module:module_doWhatYouDo()
	end		
	self.isAdjusting = false
	-- Wake 'em all up and grab current targets	
	self:UpdateAllUnits("OnEnable")
end
																													-- ------- Opts & Controls
function GroupTargets:ldb_ObjectCreate()
	return LibStub("LibDataBroker-1.1"):NewDataObject(AddonName, {
		type = 'launcher',
		label = AddonName, 
		text = "Config", 	
		icon = "Interface/Icons/ability_hunter_mastermarksman",
			-- can have EITHER OnTooltipShow OR OnEnter, not both --> OnTooltipShow = function(tooltip) end				
		OnTooltipShow = function(self) 
			GameTooltip:AddLine("Left Click to Load Options Dialog")
		end, -- self in this context is the ldbObject		
				
		OnClick = function(self, button)
			if button == "RightButton" then
	--			gtSF:toggle_StyletMode()
			else
				if not IsAddOnLoaded(AddonName.."_Config") then 
					LoadAddOn(AddonName.."_Config") 
					GroupTargetsConfig:regOptions(AddonName, GroupTargets.db.global)
				end
				GroupTargetsConfig:OpenConfig(GroupTargets.db.global)
			end
		end,
	})
end

function GroupTargets:addBlizzardOptionsPanel()
	local blizzy_Opts = {		
		type = "group",
		order = 10,
		name = ("\124T%s:%d:%d:1:0\124t %s"):format("Interface/Icons/ability_hunter_mastermarksman", 24, 24, AddonName),
		args = {
			introspacer = { order = 4, type = "description", name = "  "}, 
			intro = {
				order = 5,
				type = "description",
				fontSize = "medium",
				name = "Display the Target the the group members, their pets, and of course, your pet has.",
			},			
			quickDirections = {
				type = "group",
				order = 10,
				name = "  ",
				inline = true,
				args = {			
					intro1 = {
						order = 1,
						type = "description",
						fontSize = "medium",
						name = "You can Left-click the name of the target to assist.",
					},
					configure = {
					  type = "execute",
						name = "Configure",
					  order = 50,
						func = function()
							if not IsAddOnLoaded(AddonName.."_Config") then 
								LoadAddOn(AddonName.."_Config") 
								GroupTargetsConfig:regOptions(AddonName, GroupTargets.db.global)
							end
							GameTooltip:Hide(); 
							InterfaceOptionsFrameCancel:Click(); 
							GameMenuButtonContinue:Click(); 
							GroupTargetsConfig:OpenConfig(GroupTargets.db.global); 				
						end, 
					},							
				}							
			},
			versionData = {
				order = 20,
				type = "description",
				fontSize = "medium",
				name = ("Addon version %s updated to WoW version %s by %s"):format(GetAddOnMetadata(AddonName, 'Version'), "8.3", "Rhianidd"),
			},			
			hideMinimapIcon = {
				order = 40, 
				type = "toggle", 
				name = "Hide Minimap Icon", 
				get = function(info) return GroupTargets.db.global.DBIconTable.hide end,
				set = function(info, value)
					GroupTargets.db.global.DBIconTable.hide = value
					if value then
						LibStub("LibDBIcon-1.0"):Hide(AddonName) 
					else 
						LibStub("LibDBIcon-1.0"):Show(AddonName) 
					end
				end,
			},				
		},
	}
	LibStub("AceConfigRegistry-3.0"):RegisterOptionsTable(AddonName, blizzy_Opts)	-- Blizzy Opts -
	LibStub("AceConfigDialog-3.0"):AddToBlizOptions(AddonName, AddonName) 	      ----------------	
end

																													-- ------- Event Responders
function GroupTargets:UpdateUnit(event, unit)

	-- hFrame is the handle for the gt_unitFrame
	-- RaidIconIndex is eponymous, defaulted to 0 unless unit..target has a raid icon
	local hFrame, RaidIconIndex, color = self.uFrames[unit], 0, self.db.global.color

	if hFrame then
		if UnitExists(unit.."target") then
			-- Set var for FontString Color
			local col 		
			if     UnitIsFriend(unit, unit.."target") then col = color.Friendly
			elseif UnitIsEnemy (unit, unit.."target") then col = color.Hostile
			else                                           col = color.Neutral
			end
 
			-- Set Target Color and Name
			hFrame.fsTargetName:SetText(UnitName(unit.."target"))
			hFrame.fsTargetName:SetTextColor(col.r or 0, col.g or 0, col.b or 0)
			
			RaidIconIndex = GetRaidTargetIndex(unit.."target")			
		else
			hFrame.fsTargetName:SetText("[no target]")
			hFrame.fsTargetName:SetTextColor(color.NoTarget.r or 0, color.NoTarget.g or 0, color.NoTarget.b or 0)
		end

		-- Raid Target Icon
		SetRaidTargetIconTexture(hFrame.icon, RaidIconIndex or 0)		
	end	
end
function GroupTargets:UpdateAllUnits(event)
	-- The RAID_TARGET_UPDATE Event passes no additional information so so everyone has to be re-evaluated
	-- Var 'event' is passed for debugging
	local enabledFrames = self.db.global.enabledFrames
	for unit, _ in pairs(enabledFrames) do self:UpdateUnit(event, unit) end	
end

																													-- ------- Basic Configuration Toggler
function GroupTargets:toggle_AdjustmentMode()
	self.isAdjusting = not self.isAdjusting
	local ctg = self.db.global

	-- unReg / reReg Unit Watch for actual unit frames to work with
	for unit, blizzyFrame in pairs(BlizzardFrameIndex) do
		if self.isAdjusting then
			self:allUnits_RegisterUnitWatch(true)
			SetRaidTargetIconTexture(self.uFrames[unit].icon, 8) 
			
			UnregisterUnitWatch(_G[blizzyFrame])
			_G[blizzyFrame]:Show()
		else
			RegisterUnitWatch(_G[blizzyFrame])
			self:allUnits_RegisterUnitWatch()
			self:UpdateAllUnits("End Adjustment Mode")
		end	
	end	
end

function GroupTargets:cfg_tryColor(col)
	local enabledFrames = self.db.global.enabledFrames
	for unit, _ in pairs(enabledFrames) do 
		local uFrame = self.uFrames[unit]
		uFrame.fsTargetName:SetTextColor(col.r or 0, col.g or 0, col.b or 0)
	end	
end
																													-- ------- Constructor Assist / Config Assist
function GroupTargets:unit_RegisterUnitWatch(frame, unit, RegUnitWatch)
	if RegUnitWatch then
		frame:SetAttribute( "type1", "target" )
		frame:SetAttribute( "unit",   unit.."target" )	
		RegisterUnitWatch(frame)
	else
		frame:SetAttribute( "type1", "assist" )
  	frame:SetAttribute( "unit",  unit )
		UnregisterUnitWatch(frame)
  	frame:Show()
	end	
end

function GroupTargets:allUnits_RegisterUnitWatch(forceUnregister)
	local enabledFrames = self.db.global.enabledFrames

	for fName, _ in pairs(enabledFrames) do
		GroupTargets:unit_RegisterUnitWatch( self.uFrames[fName], fName, forceUnregister==nil and self.db.global.RegisterUnitWatch or false)
	end	
end

function GroupTargets:allUnits_Reposition()
	local pos = self.db.global.pos

	local enabledFrames = self.db.global.enabledFrames
	for unit, _ in pairs(enabledFrames) do 
		local points = pos[ SetIndex[unit] ]
		local uFrame = self.uFrames[unit]

		uFrame:ClearAllPoints()
		uFrame:SetPoint( points.point, _G[ BlizzardFrameIndex[unit] ], points.relative, points.dx, points.dy )
	end	
end
																													-- ------- Frame Constructors
function GroupTargets:styleFrame(f) -- sets defaults
	local defaults = self.db.global.defaults
	
	f:SetWidth(defaults.width)
	f:SetHeight(defaults.height)
	
	f:SetBackdrop({bgFile = defaults.fileBackdrop, edgeFile = defaults.fileEdge})
	f:SetBackdropBorderColor( defaults.bgColor, defaults.bgColor, defaults.bgColor, defaults.bgColor ) 	 -- all 0
	f:SetBackdropColor( defaults.edgeColor, defaults.edgeColor, defaults.edgeColor, defaults.edgeColor ) -- all 0
	-- Debugging Mask
	--f:SetBackdrop({bgFile = "Interface/Tooltips/UI-Tooltip-Background", edgeFile = "Interface/Tooltips/UI-Tooltip-Border",tile = true, tileSize = 16, edgeSize = 16, insets = { left = 4, right = 4, top = 4, bottom = 4 }})
	--f:SetBackdropColor(0, 1, 0, .75)

	f.icon:ClearAllPoints()
	f.icon:SetPoint( defaults.iconSide, f, defaults.iconSide, defaults.icon_dx, defaults.icon_dy )
	f.icon:SetSize( defaults.iconSize, defaults.iconSize )

	-- make room for the icon
	f.fsTargetName:ClearAllPoints()
	f.fsTargetName:SetPoint("LEFT", f, "LEFT", (defaults.icon_dx + defaults.iconSize + defaults.fsLeftAnchor_dx), 0)

	f.fsTargetName:SetFont(defaults.fsFont, defaults.fsFontSize, defaults.fsFlags)
	-- Only one anchor (LEFT) in default
	-- f.fsTargetName:SetPoint("RIGHT", f, "RIGHT", defaults.fsLeftAnchor_dx, 0)
end	
																																			
function GroupTargets:makeFrame(unit, objName, parent_frame)
	-- target-of-target button
	local f = CreateFrame("Button", objName, parent_frame, "SecureActionButtonTemplate")
	f:RegisterForClicks("AnyUp")
	f:SetMovable(true)	
	f:SetFrameStrata("BACKGROUND")

	-- raid target icon
	f.icon = f:CreateTexture()
	f.icon:SetTexture("Interface/TargetingFrame/UI-RaidTargetingIcons")
	SetRaidTargetIconTexture(f.icon, 0)

	-- fontstring for target name
	f.fsTargetName = f:CreateFontString(nil, "OVERLAY") -- "SystemFont_Shadow_Med1"={"Fonts\FRIZQT__.TTF", "12", "OUTLINE"}	
	f.fsTargetName:SetWordWrap(false)
	
	-- if gtSF is loaded, its module:DoWhatYouDo() will set the style
	if not IsAddOnLoaded(self.moduleName_STYLEFRAME) then self:styleFrame(f) end
	
	return f;
end
