--[[
	GroupTargets Config
	Version 8.3
	Last Updated:   Sept 5, 2019

	Author: Rhianidd of Silver Hand (rhianidd@silverdaggers.net)
	Website: http://www.wowinterface.com/downloads/info16413-RhianiddsGroupTargets.html
--]]

GroupTargetsConfig = LibStub("AceAddon-3.0"):NewAddon("GroupTargetsConfig", "AceEvent-3.0", "libPortableTools-1.0")
GroupTargetsConfig.Dialog = LibStub("AceConfigDialog-3.0")
local debug = function(...) GroupTargets:debugAlert(...) end

function GroupTargetsConfig:OnInitialize() end 
function GroupTargetsConfig:OnEnable()     end 

function GroupTargetsConfig:mkLink(path, size) return ("\124T"..path..":%d:%d:1:0\124t"):format(size, size) end

function GroupTargetsConfig:OpenConfig(cfg)
	local opts = self.opts
	self.Dialog:Open(opts.f_reg)
	self.Dialog:SelectGroup(opts.f_reg, opts.f_MainOpts)
end

function GroupTargetsConfig:regOptions(AddonName, cfg)
	-- cfg=GroupTargets.db.global
	local GT = GroupTargets -- make the name for function calls a tad easier to work with
	local resPool = ({GetScreenResolutions()})
	local vMax, hMax = resPool[#resPool]:match("([%d%.]*)x([%d%.]*)"); vMax=tonumber(vMax); hMax=tonumber(hMax)

	-- Options Tables 
	About = {
		type = "group",
		order = 10,
		name = self:mkLink("Interface/Icons/ability_hunter_mastermarksman", 16).." "..AddonName,
		args = {
			header = {
				order = 0, 
				type = "description", 
				fontSize = "large",
				name = "General Options",
			},
			intro = {
				order = 1,
				type = "description",
				fontSize = "large",
				name = "\nNota Bene: Some configuration options may cause errors if used during combat.",
			},
			introspacer = { order = 2, type = "description", name = "  "}, 
			versionData = {
				order = 3,
				type = "description",
				fontSize = "medium",
				name = ("Addon version %s updated to WoW version %s by %s"):format(GetAddOnMetadata(AddonName, 'Version'), "8.3", "Rhianidd"),
			},
				
			ldbAnchors = {
		    type = "group", 
		    inline = true, 
		    name = " ", 
		    order = 10,
				args = {
					hideMinimapIcon = {
						order = 20, 
						type = "toggle", 
						name = "Hide Minimap Icon", 
						get = function(info) return cfg.DBIconTable.hide end,
						set = function(info, value)
							cfg.DBIconTable.hide = value
							if value then
								LibStub("LibDBIcon-1.0"):Hide(AddonName) 
							else 
								LibStub("LibDBIcon-1.0"):Show(AddonName) 
							end
						end,
					},				
				},
			},
		},
	}
	
	mainOpts = {
		type = "group",
		order = 10,
		name = self:mkLink("Interface/Icons/ability_hunter_mastermarksman", 16).."Core Options",
		args = {
			header = {
				order = 0, 
				type = "description", 
				fontSize = "large",
				name = "Addon Configuration Options",
			},
			Colors = {
		    type = "group", 
		    inline = true, 
		    name = "Colors", 
		    order = 10,			
				args = {			
					color_FriendlyTarget = {
					  type = "color",
					  name = "Friendly Target",
					  order = 10,						
					  get = function(info)
					      local col = cfg.color.Friendly
					      return col.r, col.g, col.b
					  end,
					  set = function(info, r, g, b)
					      local col = cfg.color.Friendly
					      col.r = r or 0
					      col.g = g or 0
					      col.b = b or 0
								GT:cfg_tryColor(col)
					  end,
					},	
					color_HostileTarget = {
					  type = "color",
					  name = "Hostile Target",
					  order = 20,						
					  get = function(info)
					      local col = cfg.color.Hostile
					      return col.r, col.g, col.b
					  end,
					  set = function(info, r, g, b)
					      local col = cfg.color.Hostile
					      col.r = r or 0
					      col.g = g or 0
					      col.b = b or 0
								GT:cfg_tryColor(col)
					  end,
					},	
					color_NeutralTarget = {
					  type = "color",
					  name = "Neutral Target",
					  order = 30,						
					  get = function(info)
					      local col = cfg.color.Neutral
					      return col.r, col.g, col.b
					  end,
					  set = function(info, r, g, b)
					      local col = cfg.color.Neutral
					      col.r = r or 0
					      col.g = g or 0
					      col.b = b or 0
								GT:cfg_tryColor(col)
					  end,
					},	
					color_NoTarget = {
					  type = "color",
					  name = "No Target",
					  order = 40,						
					  get = function(info)
					      local col = cfg.color.NoTarget
					      return col.r, col.g, col.b
					  end,
					  set = function(info, r, g, b)
					      local col = cfg.color.NoTarget
					      col.r = r or 0
					      col.g = g or 0
					      col.b = b or 0
								GT:cfg_tryColor(col)
					  end,
					},
				},
			},
			RegisterUnitWatch = {
		    type = "group", 
		    inline = true, 
		    name = "Unit Display Method", 
		    order = 20,			
				args = {
					RegisterUnitWatch = {
						order = 10, 
						type = "toggle", 
						name = "Hide Target Frame when Unit has no Target", 
						width = "double",
						get = function(info) return cfg.RegisterUnitWatch end,
						set = function(info, value) 
							cfg.RegisterUnitWatch = value
							local enabledFrames = cfg.enabledFrames
							GT:allUnits_RegisterUnitWatch()
					  end,
					},
				},
			},
			unitframes = {
		    type = "group", 
		    inline = true, 
		    name = "Unit Frames (requires reload)", 
		    order = 30,
				args = {
					pet = {
						order = 10, 
						type = "toggle", 
						name = "Pet", 
						get = function(info) return cfg.enabledFrames.pet end,
						set = function(info, value) cfg.enabledFrames.pet = value and true or nil end,
					},
					party = {
						order = 20, 
						type = "toggle", 
						name = "Party Members", 
						get = function(info) return cfg.enabledSets.party end,
						set = function(info, value) 
							local v=value and true or nil
							cfg.enabledSets.party = v
							cfg.enabledFrames.party1 = v
							cfg.enabledFrames.party2 = v
							cfg.enabledFrames.party3 = v
							cfg.enabledFrames.party4 = v
						end,
					},
					partypet = {
						order = 30, 
						type = "toggle", 
						name = "Party Pets", 
						get = function(info) return cfg.enabledSets.partypet end,
						set = function(info, value) 
							local v=value and true or nil
							cfg.enabledSets.partypet = v
							cfg.enabledFrames.partypet1 = v
							cfg.enabledFrames.partypet2 = v
							cfg.enabledFrames.partypet3 = v
							cfg.enabledFrames.partypet4 = v
						end,
					},
				},			
			},
			pos = {
		    type = "group", 
		    inline = true, 
		    name = "Reposition Windows", 
		    order = 60,			
				args = {
					msg = {
						type = "description",
						order = 10,
						name = "Repositioning is done by type with pilot frames -- adust the pilot frame and the others in the group will follow suit. When in Reposition Mode, three translucent handle will appear for the respective pilot frames, drag these handles. Nota Bene: Do not attempt this during combat nor enter combat while in Reposition Mode.",
					},
					Reposition = {
	          type = "execute",
	          name = "Toggle Reposition Mode",
	          disabled = function()
	          	if IsAddOnLoaded(GT.moduleName_STYLEFRAME) then 
	          			 return gtSF.isStyling
	          	else return false
	          	end
	          end,
	          func = function()
	          	if InCombatLockdown() then
	          		GroupTargets:alert("Reposition Mode cannot be toggled while in combat.")
	          	else
	          		GroupTargets:toggle_AdjustmentMode()
	          	end
	          end,
	          order = 40
					},

					petmsg = {type = "header",order = 50,name = "Pet"},
					petxOff = {
					  type = "range",
					  name = "xOff",
					  desc = "xOff",
					  disabled = function() return not GroupTargets.isAdjusting end,
					  order = 60,
					  step = 1, min = -hMax, max = hMax,
					  isPercent = false,
						get = function() return cfg.pos.pet.dx end,
						set = function(i, v)
							cfg.pos.pet.dx = v  
							GT:allUnits_Reposition()
						end,
					},  					
					petyOff = {
					  type = "range",
					  name = "yOff",
					  desc = "yOff",
					  disabled = function() return not GroupTargets.isAdjusting end,
					  order = 70,
					  step = 1, min = -vMax, max = vMax,
					  isPercent = false,
						get = function() return cfg.pos.pet.dy end,
						set = function(i, v)
							cfg.pos.pet.dy = v 
							GT:allUnits_Reposition()
						end,
					},  					

					partymsg = {type = "header",order = 80,name = "Party"},
					partyxOff = {
					  type = "range",
					  name = "xOff",
					  desc = "xOff",
					  disabled = function() return not GroupTargets.isAdjusting end,
					  order = 90,
					  step = 1, min = -hMax, max = hMax,
					  isPercent = false,
						get = function() return cfg.pos.party.dx end,
						set = function(i, v)  
							cfg.pos.party.dx = v
							GT:allUnits_Reposition()
						end,
					},  					
					partyyOff = {
					  type = "range",
					  name = "yOff",
					  desc = "yOff",
					  disabled = function() return not GroupTargets.isAdjusting end,
					  order = 100,
					  step = 1, min = -vMax, max = vMax,
					  isPercent = false,
						get = function() return cfg.pos.party.dy end,
						set = function(i, v)  
							cfg.pos.party.dy = v
							GT:allUnits_Reposition()
						end,
					},  	

					ppmsg = {type = "header",order = 110,name = "Party Pets"},
					ppxOff = {
					  type = "range",
					  name = "xOff",
					  desc = "xOff",
					  disabled = function() return not GroupTargets.isAdjusting end,
					  order = 120,
					  step = 1, min = -hMax, max = hMax,
					  isPercent = false,
						get = function() return cfg.pos.partypet.dx end,
						set = function(i, v)
							cfg.pos.partypet.dx = v  
							GT:allUnits_Reposition()
						end,
					},  					
					ppyOff = {
					  type = "range",
					  name = "yOff",
					  desc = "yOff",
					  disabled = function() return not GroupTargets.isAdjusting end,
					  order = 130,
					  step = 1, min = -vMax, max = vMax,
					  isPercent = false,
						get = function() return cfg.pos.partypet.dy end,
						set = function(i, v)  
							cfg.pos.partypet.dy = v
							GT:allUnits_Reposition()
						end,
					},  	
				},
			},			
		},
	}

	amalgam = { -- glue-point for internal and modular opt tables
		type = "group", 
		childGroups = "tab",
	  name = AddonName, 
		args = {
			About    = About,
			mainOpts = mainOpts,
		}			
	}

	-- All modules will load a config, even if it's just a tab with an anounce, so the user is assured it's enabled/running
	local RegOpts = LibStub("AceConfigRegistry-3.0")
	for name, module in GroupTargets:IterateModules() do
		amalgam.args[name] = RegOpts:GetOptionsTable(name, "dialog", "AceConfigRegistry-3.0")
	end

	-- Register Options Tables
	self.opts = {f_reg = "GroupTargetsConfig", f_About = "About", f_MainOpts = "mainOpts"}
 	local opts = self.opts
	RegOpts:RegisterOptionsTable( opts.f_reg, amalgam ) -- GroupTargets Opts -
	self.Dialog:SetDefaultSize(   opts.f_reg, 800, 700 ) ---------------------
end 
