-- ------------------------------------------------------------------------- --
-- Project: Group Targets Health Display
-- Author:  Rhianidd
--	Version 8.3
--	Last Updated:   Sept 5, 2019
-- ------------------------------------------------------------------------- --
local AddonName = ...
local MODULE = "HealthDisplay"
local hGroupTargets = LibStub("AceAddon-3.0"):GetAddon("GroupTargets")
gtHB = hGroupTargets:NewModule(MODULE, "libPortableTools-1.0","AceEvent-3.0")
gtHB:SetEnabledState(true)
gtHB.Dialog = LibStub("AceConfigDialog-3.0")

local debug = function(...) gtHB:debugAlert(...) end
local relPoints = {"CENTER","TOP","BOTTOM","RIGHT","LEFT","TOPRIGHT","TOPLEFT","BOTTOMRIGHT","BOTTOMLEFT"}
local availSets =  {"pet", "party", "partypet"}
local textJustifyTbl = {"LEFT","RIGHT", "CENTER"}

gtHB.pcFS = {}; gtHB.HB = {}

function gtHB:OnInitialize()
	self:pt_Setup("Health Display", "gtHD", true) -- portable tools setup (debug/alert)
  -- Defaults -------------------------------------------------------------- --
	self.DBDefaults = {
   	["global"] = {
	   	["pet"] = {
	   		["me"] = true,
				["healthShow"] =  true,
				["HBPos"] = 3,
				["hbColor"] = {["r"] = 0, ["g"] = 1, ["b"] = 0},
			},
 	   	["party"] = {
	   		["me"] = true,
 				["healthShow"] =  true,
				["HBPos"] = 3,
				["hbColor"] = {["r"] = 0, ["g"] = 1, ["b"] = 0},
			},
	   	["partypet"] = {
	   		["me"] = true,
				["healthShow"] =  true,
				["HBPos"] = 3,
				["hbColor"] = {["r"] = 0, ["g"] = 1, ["b"] = 0},
			},
    },
   }
	self.db = LibStub("AceDB-3.0"):New("HealthDisplay_DB", self.DBDefaults, true)    
end

function gtHB:OnEnable() 
	self:makeHealthPercentDisplay()

	self:RegisterEvent("UNIT_MAXHEALTH", "updateUnitBar")
	self:RegisterEvent("UNIT_HEALTH", "updateUnitBar")
	self:RegisterEvent("UNIT_TARGET",	"updateUnitBar") 		 -- Fires when a unit changes targets (var0=event, var1=unit)

	self:updateUnitBar()

	self:regOptions()
end
			
function gtHB:makeHealthPercentDisplay()
	local hbStyle = self.db.global

	local enabledFrames = GroupTargets.db.global.enabledFrames
	for unit, _ in pairs(enabledFrames) do		
		self.HB[unit] = CreateFrame("Frame", unit.."targetHB",GroupTargets.uFrames[unit]);
		self.HB[unit]:SetSize(GroupTargets.uFrames[unit]:GetWidth(), 5)
		self.HB[unit]:SetBackdrop({bgFile = "Interface/Tooltips/UI-Tooltip-Background"})
		self.HB[unit]:ClearAllPoints()
		self.HB[unit]:SetPoint("TOPLEFT", self.HB[unit]:GetParent(), "BOTTOMLEFT")

		self.pcFS[unit] = self.HB[unit]:CreateFontString(nil, "OVERLAY")
		self.pcFS[unit]:SetFontObject("GameFontNormalSmall")
		self.pcFS[unit]:SetText("xxx%")
		self.pcFS[unit]:SetTextColor(1, 1, 1, 1)
		self.pcFS[unit]:ClearAllPoints()
	end	
end
			
function gtHB:updateUnitBar(event, u) 
	local enabledFrames = GroupTargets.db.global.enabledFrames
	local H
	
	for unit, _ in pairs(enabledFrames) do
		if GroupTargets.isAdjusting == true then
			self.pcFS[unit]:SetText("60%")
			self.HB[unit]:SetSize(GroupTargets.uFrames[unit]:GetWidth() * 60/100, 5)			
		elseif UnitExists(unit) and UnitExists(unit.."target") then
			H = gtHB_UpdateHealth(unit.."target")
			self.pcFS[unit]:SetText(H>0 and ("%d%%"):format(H) or "0%")
			self.HB[unit]:SetSize((GroupTargets.uFrames[unit]:GetWidth() * H/100) or 1, 5)
		else
			self.pcFS[unit]:SetText("0%")
			self.HB[unit]:SetSize(1, 5)		
		end
	end	

end

function gtHB_UpdateHealth(unittarget)
	local health = UnitHealth(unittarget);
	local maxHealth = UnitHealthMax(unittarget);
	local Hv = 0

	if maxHealth > 0 and health > 0 then
		if health > maxHealth then health = maxHealth end -- keeps bullshit values from Blizzard from making it go > 100%
		Hv = ((health * 100) / maxHealth) ;
	end
	
	return Hv;
end

															-- ------- Options
function gtHB:argInjector(name, order)
	local hbStyle = self.db.global
	local LSM = LibStub("LibSharedMedia-3.0")
	local resPool = ({GetScreenResolutions()})
	local vMax, hMax = resPool[#resPool]:match("([%d%.]*)x([%d%.]*)"); vMax=tonumber(vMax); hMax=tonumber(hMax)

	return {
				type = "group",
				name = name,
				disabled = function() return not gtHB.isStyling end,
				order = order,
				args = {				
					hbStyleMe = {
					  type = "toggle",
					  name = "Use This Style",
					  desc = "Use This Style",
					  order = 10,
						get = function(info) return hbStyle[ info[2] ].me end,
						set = function(info, v) hbStyle[ info[2] ].me = v; self:applyAllStyles()  end,
					},

					SelectCopyDestination = {
					  type = "select",
					  name = "Copy this Style To Selected",
					  desc = "Copy this Style To Selected",
					  order = 12,
					  values = availSets,
						get = function(info) return self.dest_CopyTo end,
						set = function(info, v) self.dest_CopyTo = v end,
					},	
					CopyToDestination = {
						type = "execute",
						name = "Copy to Selected",
						order = 14,
						func = function(info)
								if availSets[self.dest_CopyTo] == info[2] then
									self:alert("...ok, if you insist...")
								else
									for key, data in pairs(hbStyle[ info[2] ]) do
										hbStyle[ availSets[self.dest_CopyTo] ][key] = data
									end

									self:applyAllStyles()
								end			
			        end,
					}, -- ----------------------------------------------------------------------- Use hbStyle / Copy hbStyle

					hdrHealth = {type="header", name="Configuration", order=20},
					healthShow = {
						type = "toggle",
						name = "Show Health",
						desc = "Show Health Bar/Percent",
						order = 100,
						get = function(info) return hbStyle[info[2]].healthShow end,
						set = function(info, v) hbStyle[info[2]].healthShow = v; self:applyAllStyles() end,
					},
					
					healthBar = {
						type = "group",
						name = "Health Bar",
		    		inline = true, 
						order = 120,
						args = {
							HBPos = {
								type = "select",
								name = "Percentage Position",
								desc = "Placement of the Percentage Text",
								values = {"None", "Right Side", "On Bar"},
								order = 1,
								get = function(info) return hbStyle[info[2]].HBPos end,
								set = function(info, v) hbStyle[info[2]].HBPos = v; self:applyAllStyles() end,
							},
							hbColor = {
								type = "color",
								name = "Health Bar Color",
								desc = "Health Bar Color",
								order = 10,
							  get = function(info)
							      local col = hbStyle[info[2]].hbColor
							      return col.r, col.g, col.b
							  end,
							  set = function(info, r, g, b)
						      local col = hbStyle[info[2]].hbColor
						      col.r = r or 0
						      col.g = g or 0
						      col.b = b or 0
									self:applyAllStyles()
							  end,
							},
						},
					},
				},				
			}	
end

function gtHB:regOptions()
	local hbStyle = self.db.global
	local LSM = LibStub("LibSharedMedia-3.0")
	
	-- module controls (see :getControls_OptionTable() for injection)
	-- Debugging Note: info[2] contains arg-level 2 (e.g., optTable_HealthFrames=1, 2=pet/party/partypet, 3=StyleMe, frameHeight, etc , ... )
	-- so the group names used are those in the set index names for implicit update (i.e., hbStyle.pet.varvar => hbStyle[ info[2] ].varvar )
	-- which allows a single table to get injected 3x, plus 3x less debugging, 3x less problem introduction spots, etc
	optTable_HealthFrames = { -- Add Control Elements to the safetable in loadGroups_toOptionsTable or the pruner WILL EAT THEM
	  type = "group", 
		order = 100,
	 	name = "Health Bars",
	 	childGroups = "tab", -- nix and unrem inline=true below
		args = {
			header = {
				order = 0, 
				type = "description", 
				fontSize = "large",
				name = "Health Bar Configuration",
			},
			
			spcr00 = {order=11,type="description",name=" "},
			togglStyle = {
	         type = "execute",
	         name = "Test Style/Health Bar",
	         func = function()
	         	if InCombatLockdown() then
	         		GroupTargets:alert("Styling Mode cannot be toggled while in combat.")
	         	else
	         		gtHB:toggle_StyletMode()

						-- If GroupTargets is repositioning when Styling is activated, we don't want to toggle repositioning off.
	         		if self.isStyling and GroupTargets.isAdjusting then
	         			-- do nothing; too lazy to write negative logic so enjoy this rem statement and tell your children about it.
	         		else
	         			GroupTargets:toggle_AdjustmentMode()
	         		end
	         	end
	         end,
	         order = 25
			},			
		},
	} 

	optTable_HealthFrames.args.pet 		 = self:argInjector("Pet", 			 30)
	optTable_HealthFrames.args.party 	 = self:argInjector("Party", 		 40)
	optTable_HealthFrames.args.partypet	 = self:argInjector("Party Pet", 50)

	-- Register Blizzy Options Tables, associate with ExecAssist's panel
	local RegOpts = LibStub("AceConfigRegistry-3.0")
	RegOpts:RegisterOptionsTable(MODULE, optTable_HealthFrames)	--------- Blizzy Opts -
end 
																													-- ------- Module Executor
function gtHB:module_doWhatYouDo() -- Main Call from GroupTargets at end of :OnEnable()
	-- Setup Health Displays
--	self:applyAllStyles()

end
																													-- ------- Stylers	
function gtHB:applyAllStyles()
	local gt = GroupTargets
	local enabledFrames = gt.db.global.enabledFrames

	for unit, _ in pairs(enabledFrames) do self:StyleFrame(gt.uFrames[unit], unit) end		
end

function gtHB:StyleFrame(frame, unit)
	local LSM = LibStub("LibSharedMedia-3.0")
	local hbStyle

	if unit:match("party") then
		if unit:match("pet") then -- partypet
			hbStyle = gtHB.db.global.partypet
		else -- party
			hbStyle = gtHB.db.global.party
		end	
	else -- pet
		hbStyle = gtHB.db.global.pet
	end
	
	if hbStyle.me then
		gtHB.HB[unit]:SetBackdropColor(hbStyle.hbColor.r, hbStyle.hbColor.g, hbStyle.hbColor.b)
		if hbStyle.healthShow then gtHB.HB[unit]:Show() else gtHB.HB[unit]:Hide() end
		
		self.pcFS[unit]:ClearAllPoints()
		if hbStyle.HBPos == 2 then -- right side
			gtHB.pcFS[unit]:Show()
			self.pcFS[unit]:SetJustifyH( "LEFT")
			gtHB.pcFS[unit]:SetPoint("LEFT", GroupTargets.uFrames[unit], "RIGHT", 3, 0)
		elseif hbStyle.HBPos == 3 then -- on bar (3)
			gtHB.pcFS[unit]:Show()
			self.pcFS[unit]:SetJustifyH( "CENTER")
			gtHB.pcFS[unit]:SetPoint("TOP", GroupTargets.uFrames[unit], "BOTTOM", 3, 0)
		else
			gtHB.pcFS[unit]:Hide() 
		end

		self:updateUnitBar()
	else
		-- Nota Bene: When this module is loaded, THIS CALL is what sets the default style on unstyled frames
		--            as a cheat, since this is call is mode-agnostic, it will essentially reset the default style
		--            right after creation so GT checks for gtSF and if loaded, doesn't style the frames at all.
		GroupTargets:styleFrame(frame)
	end

end
																													-- ------- Mode Toggler
function gtHB:toggle_StyletMode()
	local ctg = self.db.global
	self.isStyling = not self.isStyling

	if self.isStyling then
		-- Force Unit Watch to OFF (true==ForceUnregister)
		GroupTargets:allUnits_RegisterUnitWatch(true)

		-- apply a Raid Icon
		local gt = GroupTargets
		local enabledFrames = gt.db.global.enabledFrames
		for unit, _ in pairs(enabledFrames) do 
			SetRaidTargetIconTexture(gt.uFrames[unit].icon, 8) 
			gt.uFrames[unit].fsTargetName:SetText("[Some Mobs Have Really, Really, Really Long Names]")
		end
		self:applyAllStyles()
	else
		-- Restore Unit Watch to Configured State
		--		GroupTargets:allUnits_RegisterUnitWatch()
		--		GroupTargets:UpdateAllUnits("End gtHB Styling")
		-- NB: These are called in GT:toggle_AdjustmentMode()
	end	
end
