-- ------------------------------------------------------------------------- --
-- Project: Group Targets Style Config
--	Author: Rhianidd
--	Version 8.3
--	Last Updated:   Sept 5, 2019
-- ------------------------------------------------------------------------- --
local AddonName = ...
local MODULE = "StyleFrames"
local hGoupTargets = LibStub("AceAddon-3.0"):GetAddon("GroupTargets")
gtSF = hGoupTargets:NewModule(MODULE, "libPortableTools-1.0")
gtSF:SetEnabledState(true)
gtSF.Dialog = LibStub("AceConfigDialog-3.0")

local debug = function(...) gtSF:debugAlert(...) end
local relPoints = {"CENTER","TOP","BOTTOM","RIGHT","LEFT","TOPRIGHT","TOPLEFT","BOTTOMRIGHT","BOTTOMLEFT"}
local availSets =  {"pet", "party", "partypet"}
local TextJustifyTbl = {"LEFT","RIGHT", "CENTER"}

function gtSF:OnInitialize()
	self:pt_Setup("Style Targets", "gtSF", true) -- portable tools setup (debug/alert)
	
  -- Defaults -------------------------------------------------------------- --
	self.DBDefaults = {
    ["global"] = {
	   	["pet"] = {
	   		["me"] = false,
				["bgHeight"] = 22,
				["bgWidth"] = 250,				
				["tile_boolean"] = true,
				["tile_size"] = 16,
				["background"] = "None",
				["border"] = "None",
				["edgesize"] = 6,
				["inset"] = 1, 
				["bgcolor"] = {["r"] = 1, ["g"] = 1, ["b"] = 1},
				["bdrcolor"] = {["r"] = 1, ["g"] = 1, ["b"] = 1},

				["font"] = "Friz Quadrata TT",
				["font_size"] = 12,
				["font_flag"] = true,
				["fs_Justify"] = 1, -- 1=LEFT (see local table var fs_justify)
				["fs_LeftPadding"] = 4,
				["fs_AnchorRightSide"] = false,
				["fs_RightPadding"] = -4,
	
				["iconsize"] = 16,
				["iconPos"] = {
					["point"] = 5, -- "LEFT",
					["AttachIconTo"] = 1, -- 1=frame, 2=fontstring
					["relPoint"] = 5, -- "LEFT",
					["xOff"] = 0,
					["yOff"] = 0
				},
    	},
	   	["party"] = {
	   		["me"] = false,
				["bgHeight"] = 22,
				["bgWidth"] = 250,				
				["tile_boolean"] = true,
				["tile_size"] = 16,
				["background"] = "None",
				["border"] = "None",
				["edgesize"] = 6,
				["inset"] = 1, 
				["bgcolor"] = {["r"] = 1, ["g"] = 1, ["b"] = 1},
				["bdrcolor"] = {["r"] = 1, ["g"] = 1, ["b"] = 1},

				["font"] = "Friz Quadrata TT",
				["font_size"] = 12,
				["font_flag"] = true,
				["fs_Justify"] = "LEFT",
				["fs_LeftPadding"] = 4,
				["fs_AnchorRightSide"] = false,
				["fs_RightPadding"] = -4,
	
				["iconsize"] = 16,
				["iconPos"] = {
					["point"] = 5, -- "LEFT",
					["AttachIconTo"] = 1, -- 1=frame, 2=fontstring
					["relPoint"] = 5, -- "LEFT",
					["xOff"] = 0,
					["yOff"] = 0
				},
    	},
	   	["partypet"] = {
	   		["me"] = false,
				["bgHeight"] = 22,
				["bgWidth"] = 250,				
				["tile_boolean"] = true,
				["tile_size"] = 16,
				["background"] = "None",
				["border"] = "None",
				["edgesize"] = 6,
				["inset"] = 1, 
				["bgcolor"] = {["r"] = 1, ["g"] = 1, ["b"] = 1},
				["bdrcolor"] = {["r"] = 1, ["g"] = 1, ["b"] = 1},

				["font"] = "Friz Quadrata TT",
				["font_size"] = 12,
				["font_flag"] = true,
				["fs_Justify"] = "LEFT",
				["fs_LeftPadding"] = 4,
				["fs_AnchorRightSide"] = false,
				["fs_RightPadding"] = -4,
	
				["iconsize"] = 16,
				["iconPos"] = {
					["point"] = 5, -- "LEFT",
					["AttachIconTo"] = 1, -- 1=frame, 2=fontstring
					["relPoint"] = 5, -- "LEFT",
					["xOff"] = 0,
					["yOff"] = 0
				},
    	},
    },
  }
  self.db = LibStub("AceDB-3.0"):New("StyleFrames_DB", self.DBDefaults, true)    
end
function gtSF:OnEnable() self:regOptions() end

																													-- ------- Options
function gtSF:argInjector(name, order)
	local style = self.db.global
	local LSM = LibStub("LibSharedMedia-3.0")
	local resPool = ({GetScreenResolutions()})
	local vMax, hMax = resPool[#resPool]:match("([%d%.]*)x([%d%.]*)"); vMax=tonumber(vMax); hMax=tonumber(hMax)

	return {
				type = "group",
				name = name,
				disabled = function() return not gtSF.isStyling end,
				-- inline = true, -- will override parent 'tabs' child behavior instruction
				order = order,
				args = {				
					styleMe = {
					  type = "toggle",
					  name = "Use This Style",
					  desc = "Use This Style",
					  order = 10,
						get = function(info) return style[ info[2] ].me end,
						set = function(info, v) style[ info[2] ].me = v; self:applyAllStyles()  end,
					},

					SelectCopyDestination = {
					  type = "select",
					  name = "Copy Style To Selected",
					  desc = "Copy Style To Selected",
					  order = 12,
					  values = availSets,
						get = function(info) return self.dest_CopyTo end,
						set = function(info, v) self.dest_CopyTo = v end,
					},	
					CopyToDestination = {
						type = "execute",
		        name = "Copy to Selected",
		        order = 14,
		        func = function(info)
							if availSets[self.dest_CopyTo] == info[2] then
								self:alert("...ok, if you insist...")
							else
								for key, data in pairs(style[ info[2] ]) do
									style[ availSets[self.dest_CopyTo] ][key] = data
								end

								self:applyAllStyles()
							end
							
		        end,
					}, -- ----------------------------------------------------------------------- Use Style / Copy Style

					hdrFrameSize = {type="header", name="Frame Size", order=20},
					frameHeight = {
						type = "range",
						name = "Frame Height",
						order = 30,
						width = "normal",
						min = 6, max = 200, step = 1,
						get = function(info) return style[ info[2] ].bgHeight end,
						set = function(info, value) style[ info[2] ].bgHeight = value; self:applyAllStyles() end,
					},		
					frameWidth = {
						type = "range",
						name = "Frame Width",
						order = 40,
						width = "normal",
						min = 6, max = 800, step = 1,
						get = function(info) return style[ info[2] ].bgWidth end,
						set = function(info, value) style[ info[2] ].bgWidth = value; self:applyAllStyles() end,
					}, -- ----------------------------------------------------------------------- Frame Size
				
					hdrFonts = {type="header", name="Fonts", order=50},
					font = { 
						type = "select",
						name = "Font",
						order = 60,
						width = "normal",
						values = LSM:List("font"),
						get = function(info) for i, v in next, LSM:List("font") do if v == style[ info[2] ].font then return i end end end,
						set = function(info, value)
							style[ info[2] ].font = LSM:List("font")[value]											
							self:applyAllStyles() 
						end,
					},
					fontSize = {
						type = "range",
						name = "Font Size",
						order = 70,
						width = "normal",
						min = 6, max = 30, step = 1,
						get = function(info) return style[ info[2] ].font_size end,
						set = function(info, value) style[ info[2] ].font_size = value; self:applyAllStyles() 
					 end,
					},
					fontFlag = { -- Outline Only so boolean until otherwise requested
					  type = "toggle",
					  name = "Outline",
					  desc = "Outline",
					  order = 75,
						get = function(info) return style[ info[2] ].font_flag end,
						set = function(info, v) style[ info[2] ].font_flag = v; self:applyAllStyles()  end,					
					},
					fontJustify = { --  ("LEFT","RIGHT", or "CENTER")
					  type = "select",
					  name = "Text Alignment",
					  desc = "Only has effect when 'Anchor Right Side' is enabled.\n\nWhen only one side is anchored, the FontString shrinks to the size of the characters, so Left/Center/Right alignment produce the same visual result.",
					  disabled = function(info) return not style[ info[2] ].fs_AnchorRightSide end,
					  order = 77,
					  values = TextJustifyTbl,
						get = function(info) return style[ info[2] ].fs_Justify end,
						set = function(info, v) style[ info[2] ].fs_Justify = v; self:applyAllStyles()  end,
					
					}, -- ----------------------------------------------------------------------- Fonts

					hdrFontStringAnchors = {type="header", name="Target Name Anchors", order=80},
					fs_LeftSidePadding = {
						type = "range",
						name = "Left Side Padding",
						order = 90,
						width = "normal",
						min = -100, max = 100, step = 1,
						get = function(info) return style[ info[2] ].fs_LeftPadding end,
						set = function(info, value) style[ info[2] ].fs_LeftPadding = value; self:applyAllStyles() end,
					},
					fs_AnchorRightSide = {
					  type = "toggle",
					  name = "Anchor Right Side",
					  desc = "Anchor Right Side",
					  order = 100,
						get = function(info) return style[ info[2] ].fs_AnchorRightSide end,
						set = function(info, v) style[ info[2] ].fs_AnchorRightSide = v; self:applyAllStyles()  end,
					},
					fs_RightSidePadding = {
						type = "range",
						name = "Right Side Padding",
						disabled = function(info) return not style[ info[2] ].fs_AnchorRightSide  end,
						order = 110,
						width = "normal",
						min = -100, max = 100, step = 1,
						get = function(info) return style[ info[2] ].fs_RightPadding end,
						set = function(info, value) style[ info[2] ].fs_RightPadding = value; self:applyAllStyles() end,
					}, -- ----------------------------------------------------------------------- Font String Anchoring

					hdrBackdrop = {type="header", name="Backdrop", order=120},
					background = {
						type = "select",
						name = "Background",
						order = 130,
						dialogControl = "LSM30_Background",
						values = LSM:HashTable("background"),
						get = function(info) return style[ info[2] ].background end,
						set = function(info, v) style[ info[2] ].background = v; self:applyAllStyles() end,
					},
					BackdropColor = {
					  type = "color",
					  name = "Color",
					  order = 140,						
					  get = function(info)
					      local col = style[ info[2] ].bgcolor
					      return col.r, col.g, col.b
					  end,
					  set = function(info, r, g, b)
					      local col = style[ info[2] ].bgcolor
					      col.r = r or 0
					      col.g = g or 0
					      col.b = b or 0
								self:applyAllStyles()
					  end,
					},	
					tile_boolean = {
					  type = "toggle",
					  name = "Tile Backdrop",
					  desc = "Tile Backdrop",
					  order = 150,
						get = function(info) return style[ info[2] ].tile_boolean end,
						set = function(info, v) style[ info[2] ].tile_boolean = v; self:applyAllStyles()  end,
					},
					tile_size = {
					  type = "range",
					  name = "Tile Size",
					  desc = "Tile Size (if Tiled)",
					  order = 160,
					  step = 1, min = 1, max = 1024,
					  isPercent = false,
						get = function(info) return style[ info[2] ].tile_size end,
						set = function(info, v) style[ info[2] ].tile_size = v; self:applyAllStyles()  end,
					}, -- ----------------------------------------------------------------------- Backdrop

					hdrBorder = {type="header", name="Border", order=170},					
					Border = {
						type = "select",
						name = "Border",
						order= 180,
						dialogControl = "LSM30_Border",
						values = LSM:HashTable("border"),
						get = function(info) return style[ info[2] ].border end,
						set = function(info, v) style[ info[2] ].border = v; self:applyAllStyles() end,								
					},
					BorderColor = {
					  type = "color",
					  name = "Color",
					  order = 190,
					  get = function(info)
					      local col = style[ info[2] ].bdrcolor
					      return col.r, col.g, col.b
					  end,
					  set = function(info, r, g, b)
					      local col = style[ info[2] ].bdrcolor
					      col.r = r or 0
					      col.g = g or 0
					      col.b = b or 0
								self:applyAllStyles()
					  end,
					},
					edgesize = {
					  type = "range",
					  name = "Edge size",
					  desc = "Edge size",
					  order = 200,
					  step = 1, min = 1, max = 30,
					  isPercent = false,
						get = function(info) return style[ info[2] ].edgesize end,
						set = function(info, v) style[ info[2] ].edgesize = v; self:applyAllStyles()  end,
					},  
					inset = {
					  type = "range",
					  name = "Inset",
					  desc = "Inset",
					  order = 210,
					  step = .5, min = 1, max = 30,
					  isPercent = false,
						get = function(info) return style[ info[2] ].inset end,
						set = function(info, v) style[ info[2] ].inset = v; self:applyAllStyles()  end,
					}, -- ----------------------------------------------------------------------- Border

					hdrRaidIcon = {type="header", name="Marker Icon", order=220},	
					iconSize = {
					  type = "range",
					  name = "iconsize",
					  desc = "iconsize",
					  order = 230,
					  step = 1, min = 1, max = 200,
					  isPercent = false,
						get = function(info) return style[ info[2] ].iconsize end,
						set = function(info, v) style[ info[2] ].iconsize = v; self:applyAllStyles()  end,
					},  
					AttachIconPoint = {
					  type = "select",
					  name = "Icon's Attach Side",
					  desc = "Icon's Attach Side",
					  order = 240,
					  values = relPoints,
						get = function(info) return style[ info[2] ].iconPos.point end,
						set = function(info, v) style[ info[2] ].iconPos.point = v; self:applyAllStyles()  end,
					},	
					AttachIconTo = {
					  type = "select",
					  name = "Object to Attach Icon To",
					  desc = "Object to Attach Icon To",
					  order = 250,
					  values = {"Frame", "Target Name"},
						get = function(info) return style[ info[2] ].iconPos.AttachIconTo end,
						set = function(info, v) style[ info[2] ].iconPos.AttachIconTo = v; self:applyAllStyles()  end,
					},	
					AttachToTargetPoint = {
					  type = "select",
					  name = "Attach Icon to Object's Side",
					  desc = "Attach Icon to Object's Side",
					  order = 260,
					  values = relPoints,
						get = function(info) return style[ info[2] ].iconPos.relPoint end,
						set = function(info, v) style[ info[2] ].iconPos.relPoint = v; self:applyAllStyles()  end,
					},	
					IconxOff = {
					  type = "range",
					  name = "xOff",
					  desc = "xOff",
					  order = 270,
					  step = 1, min = -hMax, max = hMax,
					  isPercent = false,
						get = function(info) return style[ info[2] ].iconPos.xOff end,
						set = function(info, v) style[ info[2] ].iconPos.xOff = v; self:applyAllStyles() end,
					},  					
					IconyOff = {
					  type = "range",
					  name = "yOff",
					  desc = "yOff",
					  order = 280,
					  step = 1, min = -vMax, max = vMax,
					  isPercent = false,
						get = function(info) return style[ info[2] ].iconPos.yOff end,
						set = function(info, v) style[ info[2] ].iconPos.yOff = v; self:applyAllStyles() end,
					}, -- ----------------------------------------------------------------------- Marker Icon
				},				
	}	
	
end
function gtSF:regOptions()
	local style = self.db.global
	local LSM = LibStub("LibSharedMedia-3.0")
debug("gtSF regOpts")	
	-- module controls (see :getControls_OptionTable() for injection)
			-- Debugging Note: info[2] contains arg-level 2 (e.g., optTable_SkyFrames=1, 2=pet/party/partypet, 3=styleMe, frameHeight, etc , ... )
			-- so the group names used are those in the set index names for implicit update (i.e., style.pet.varvar => style[ info[2] ].varvar )
			-- which allows a single table to get injected 3x, plus 3x less debugging, 3x less problem introduction spots, etc
	optTable_SkyFrames = { -- Add Control Elements to the safetable in loadGroups_toOptionsTable or the pruner WILL EAT THEM
	  type = "group", 
		order = 100,
	 	name = "Style",
	 	childGroups = "tab", -- nix and unrem inline=true below
		args = {
			header = {
				order = 0, 
				type = "description", 
				fontSize = "large",
				name = "Target Styling",
			},
			intro = {
				order = 10,
				type = "description",
				fontSize = "medium",
				name = "Choose the style for the named target displays",			
			},			
			spcr00 = {order=11,type="description",name=" "},
			togglStyle = {
         type = "execute",
         name = "Toggle Styling Mode",
         func = function()
         	if InCombatLockdown() then
         		GroupTargets:alert("Styling Mode cannot be toggled while in combat.")
         	else
         		gtSF:toggle_StyletMode()

						-- If GroupTargets is repositioning when Styling is activated, we don't want to toggle repositioning off.
         		if self.isStyling and GroupTargets.isAdjusting then
         			-- do nothing; too lazy to write negative logic so enjoy this rem statement and tell your children about it.
         		else
         			GroupTargets:toggle_AdjustmentMode()
         		end
         	end
         end,
         order = 25
			},
			
			ShowIcon = {
			  type = "select",
			  name = "Select Icon to Show",
			  desc = "Select Icon to Show",
			  disabled = function() return not gtSF.isStyling end,
			  order = 26,
			  values = {"Star", "Circle", "Diamond", "Triangle", "Moon", "Square", "X", "Skull", "None" },
				get = function(info) if not self.showicon then showicon=8 end; return self.showicon end,
				set = function(info, v) 
					self.showicon = tonumber(v)
					local enabledFrames = GroupTargets.db.global.enabledFrames
					for unit, _ in pairs(enabledFrames) do 
						SetRaidTargetIconTexture(GroupTargets.uFrames[unit].icon, self.showicon or 8)
					end	
				end,
			}
		},
	} 

	optTable_SkyFrames.args.pet 		 = self:argInjector("Pet", 			 30)
	optTable_SkyFrames.args.party 	 = self:argInjector("Party", 		 40)
	optTable_SkyFrames.args.partypet = self:argInjector("Party Pet", 50)

	-- Register Blizzy Options Tables, associate with ExecAssist's panel
	local RegOpts = LibStub("AceConfigRegistry-3.0")
	RegOpts:RegisterOptionsTable(MODULE, optTable_SkyFrames)	--------- Blizzy Opts -
	
end 
																													-- ------- Module Executor
function gtSF:module_doWhatYouDo() -- Main Call from GroupTargets at end of :OnEnable()
	-- Apply Styles
	self:applyAllStyles()

end
																													-- ------- Stylers	
function gtSF:applyAllStyles()
	local gt = GroupTargets
	local enabledFrames = gt.db.global.enabledFrames

	for unit, _ in pairs(enabledFrames) do self:styleFrame(gt.uFrames[unit], unit) end		
end
function gtSF:styleFrame(frame, unit)
	local LSM = LibStub("LibSharedMedia-3.0")
	local style

	if unit:match("party") then
		if unit:match("pet") then -- partypet
			style = gtSF.db.global.partypet
		else -- party
			style = gtSF.db.global.party
		end	
	else -- pet
		style = gtSF.db.global.pet
	end

	if style.me then
		-- Resize Frame
		frame:SetSize(style.bgWidth, style.bgHeight )

		-- Background / Edge
		frame:SetBackdrop({
			bgFile = AceGUIWidgetLSMlists.background[style.background],
			edgeFile = AceGUIWidgetLSMlists.border[style.border],
			tile = style.tile_boolean, tileSize = style.tile_size, edgeSize = style.edgesize, insets = {left = style.inset, right = style.inset, top = style.inset, bottom = style.inset}
	   })
		frame:SetBackdropColor(style.bgcolor.r, style.bgcolor.g, style.bgcolor.b, 1)
		frame:SetBackdropBorderColor(style.bdrcolor.r, style.bdrcolor.g, style.bdrcolor.b, 1)
		
		frame.icon:SetSize(style.iconsize, style.iconsize)
		frame.icon:ClearAllPoints()
		frame.icon:SetPoint(
			relPoints[style.iconPos.point],
			style.iconPos.AttachIconTo==1 and frame or frame.fsTargetName,
			relPoints[style.iconPos.relPoint],
			style.iconPos.xOff,
			style.iconPos.yOff
		)

		frame.fsTargetName:SetFont(LSM:Fetch("font", style.font), style.font_size, style.font_flag and "OUTLINE" or "")
		frame.fsTargetName:SetJustifyH( TextJustifyTbl[style.fs_Justify] )
		
		frame.fsTargetName:ClearAllPoints()
		frame.fsTargetName:SetPoint("LEFT", frame, "LEFT", style.fs_LeftPadding, 0)
		if style.fs_AnchorRightSide then
			frame.fsTargetName:SetPoint("RIGHT", frame, "RIGHT", style.fs_RightPadding, 0)	
		end
		
	else
		-- Nota Bene: When this module is loaded, THIS CALL is what sets the default style on unstyled frames
		--            as a cheat, since this is call is mode-agnostic, it will essentially reset the default style
		--            right after creation so GT checks for gtSF and if loaded, doesn't style the frames at all.
		GroupTargets:styleFrame(frame)
	end
end
																													-- ------- Mode Toggler
function gtSF:toggle_StyletMode()
	local ctg = self.db.global
	self.isStyling = not self.isStyling

	if self.isStyling then
		-- Force Unit Watch to OFF (true==ForceUnregister)
		GroupTargets:allUnits_RegisterUnitWatch(true)

		-- apply a Raid Icon
		local gt = GroupTargets
		local enabledFrames = gt.db.global.enabledFrames
		for unit, _ in pairs(enabledFrames) do 
			SetRaidTargetIconTexture(gt.uFrames[unit].icon, 8) 
			gt.uFrames[unit].fsTargetName:SetText("[Some Mobs Have Really, Really, Really Long Names]")
		end

	else
		-- Restore Unit Watch to Configured State
		--		GroupTargets:allUnits_RegisterUnitWatch()
		--		GroupTargets:UpdateAllUnits("End gtSF Styling")
		-- NB: These are called in GT:toggle_AdjustmentMode()
	end	

end
